#!/usr/bin/env python

# Timer
class Timer():
    """
    计算函数, 段落的执行时间
    ex:
        def example():
            Timer.start()
            time.sleep(1)
            Timer.end()

        @Timer.calc
        def example():
            time.sleep(1)
    """

    logger = ''
    start_time = 0
    end_time = 0

    def __init__(self):
        pass

    @classmethod
    def start(cls):
        cls.start_time = time.time()
        msg = 'Caculate time start'
        if cls.logger:
            extra = {
                'pathname': sys._getframe(1).f_code.co_filename,
                'lineno': sys._getframe(1).f_lineno,
            }
            cls.logger.info(msg, extra=extra)
        else:
            print(msg)

    @classmethod
    def end(cls):
        cls.end_time = time.time()
        msg = 'Caculate time end, Time: %0.2f sec'%(cls.end_time - cls.start_time)
        if cls.logger:
            extra = {
                'pathname': sys._getframe(1).f_code.co_filename,
                'lineno': sys._getframe(1).f_lineno,
            }
            cls.logger.info(msg, extra=extra)
        else:
            print(msg)

    @classmethod
    def calc(cls, func):
        def wrap(*args, **kwargs):
            start_time = time.time()
            rsp = func(*args, **kwargs)
            end_time = time.time()
            msg = 'Time: %0.2f sec'%(end_time - start_time)
            if cls.logger:
                cls.logger.info(msg)
            else:
                print(msg)
            return rsp
        return wrap




#!/usr/bin/env python

"""
JSON 默認編碼改寫
"""

import json
from datetime import datetime
from json import JSONEncoder

class JSONEncoder(json.JSONEncoder):

    def default(self, obj):
        if isinstance(obj, datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        base_type = (str, int, float, dict, list)
        if (not isinstance(obj, base_type) and
                hasattr(obj, '__str__')):
            return str(obj)
        raise TypeError(f'Object of type {val.__class__.__name__} '
                                        f'is not JSON serializable')
# 調整默認值
def dumps(obj, *, skipkeys=False, ensure_ascii=False, check_circular=True,
        allow_nan=True, cls=None, indent=None, separators=None,
        default=None, sort_keys=False, **kw):

    if cls is None:
        cls = JSONEncoder
    return cls(
        skipkeys=skipkeys, ensure_ascii=ensure_ascii,
        check_circular=check_circular, allow_nan=allow_nan, indent=indent,
        separators=separators, default=default, sort_keys=sort_keys,
        **kw).encode(obj)

json.dumps = dumps



#!/usr/bin/env python

"""
author: walker
description:
    Python 工具包
change log:
    2019-12-29 init
    2020-01-17 Add Requests Class
    2020-02-01 Add logrotate Function
    2020-04-19 增加Param類型判斷
    2021-01-19 調整json默認編碼
"""

import io
import os
import sys
import time
import shutil
import requests
import traceback
import prettytable as pt
from datetime import datetime, timedelta

from . import logging
from .mail import Mail
from .timer import Timer
from .json import json
from .encryption import pass_encode, pass_decode

def logrotate(target, interval=timedelta(days=1), time_format='%Y%m%d'):
    """
    日志切割
    :param target(str): 目标文件夹or日志
    :param interval(timedelta): 切割间隔
    :param time_format(str): 文件名日期格式
    :return: None
    """
    if os.path.isfile(target):
        path = ''
        files = [target]
    else:
        path = target if target[-1] == '/' else target + '/'
        files = os.listdir(target)

    for f in files:
        segment = f.split('.')
        if len(segment) != 2 or segment[-1] != 'log':
            continue
        flag = (datetime.now() - interval).strftime(time_format)
        src = path + f
        dst = path + '{}.{}.{}'.format(segment[0], flag, segment[1])
        if os.path.exists(dst):
            continue
        shutil.copyfile(src, dst)
        with open(src, 'w') as f:
            pass

def print_table(header, data, sort=None):
    """
    字典列表，输出成表格形式
    """
    tb = pt.PrettyTable()
    tb.field_names = header
    for d in data:
        tb.add_row([d[h] for h in header])
    tb.align = 'l'
    if sort is not None:
        tb.sortby = header[sort]
    print(tb)

# Format Request
class Requests():
    """
    包裝request方法，
    只支援JSON格式，
    return Result
    """
    def __init__(self, *args, **kwargs):
        # 取消告警
        requests.packages.urllib3.disable_warnings()

        self.header = {
            'Content-Type': 'application/json',
        }
        self.timeout = kwargs.get('timeout', 10)
        self.result = Result()

    def get(self, url, body={}):
        """
        通用Get方法
        """
        try:
            resp = requests.get(url, json=body, headers=self.header, 
                    timeout=self.timeout, verify=False)
            if resp.status_code != 200:
                self.result.set(resp.text, False)
                return self.result
            self.result.set(json.loads(resp.text))
        except json.decoder.JSONDecodeError:
            self.result.set(resp.text, False)
        except Exception as e:
            self.result.set(str(e), False)
        return self.result

    def post(self, url, body=None):
        """
        通用Post方法
        """
        try:
            resp = requests.post(url, json=body, headers=self.header,
                    timeout=self.timeout, verify=False)
            if resp.status_code not in [200, 201, 202, 204]:
                self.result.set(resp.text, False)
                return self.result
            self.result.set(json.loads(resp.text))
        except json.decoder.JSONDecodeError:
            self.result.set(resp.text, False)
        except Exception as e:
            self.result.set(str(e), False)
        return self.result

    def put(self, url, body=None):
        """
        通用Put方法
        """
        try:
            resp = requests.put(url, json=body, headers=self.header,
                    timeout=self.timeout, verify=False)
            if resp.status_code not in [200, 201, 202, 204]:
                self.result.set(resp.text, False)
                return self.result
            self.result.set(json.loads(resp.text))
        except json.decoder.JSONDecodeError:
            self.result.set(resp.text, False)
        except Exception as e:
            self.result.set(str(e), False)
        return self.result

    def delete(self, url, body=None):
        """
        通用Delete方法
        """
        try:
            resp = requests.delete(url, json=body, headers=self.header,
                    timeout=self.timeout, verify=False)
            if resp.status_code not in [200, 201, 202, 204]:
                self.result.set(resp.text, False)
                return self.result
            if resp.text:
                self.result.set(json.loads(resp.text))
            else:
                self.result.set('')
        except json.decoder.JSONDecodeError:
            self.result.set(resp.text, False)
        except Exception as e:
            self.result.set(str(e), False)
        return self.result

# Parse Argument
class Param():
    """
    参数校验
    """
    def __init__(self, data):
        self._data = data
        self.args = {}

    def __getitem__(self, key):
        return self.args[key]

    def __setitem__(self, key, val):
        self.args[key] = val

    def __len__(self):
        return len(self.args)

    def parse(self, key, default='__placeholder__', type=None):
        """
        参数解析
        :param key(any): 参数
        :param defualt(any): 默认值
        :param type(type): 指定参数类型
        """
        arg = ''
        msg = 'Missing parameters {}'.format(key)
        if key in self._data:
            if self._data[key] == '':
                if default == '__placeholder__':
                    raise ValueError(msg)
                arg = self.define_type(default, type)
            else:
                arg = self.define_type(self._data[key], type)
        else:
            if default == '__placeholder__':
                raise ValueError(msg)
            arg = self.define_type(default, type)
        self.args[key] = arg
        setattr(self, key, arg)

    def define_type(self, value, type):
        """
        类型校验
        """
        msg = 'Type parameter Error'
        if type is None:
            return value
        if type in [str, int, float]:
            return type(value)
        if type in [bool]:
            if value in [0, '', 'False', 'false', False]:
                return False
            else:
                return True
        if type in [list, dict]:
            if isinstance(value, type):
                return value
        raise TypeError(msg)

# Format Result
class Result():
    """
    函数执行结果格式化
    OK: 是否成功(True|False)
    line: 代码哪一行输入
    data: 输出的结果
    time: 从初始化到设值的时间
    """
    OK = True
    line = ''
    data = ''
    time = 0

    def __init__(self):
        self.set_dict()
        self._time = time.time()

    def __str__(self):
        if isinstance(self.data, str):
            return self.data
        elif isinstance(self.data, list):
            return json.dumps(self.data, indent=2, ensure_ascii=False)
        elif isinstance(self.data, dict):
            return json.dumps(self.data, indent=2, ensure_ascii=False)
        else:
            return str(self.data)

    def __repr__(self):
        return self.__str__()

    def set(self, data, OK=True):
        filename = sys._getframe(1).f_code.co_filename
        line = sys._getframe(1).f_lineno

        self.OK = OK
        self.line = '{}(line:{})'.format(filename, line)
        self.data = data
        self.time = '%0.3f'%(time.time() - self._time)
        self.set_dict()

    def set_dict(self):
        self.dict = {
            'OK': self.OK,
            'line': self.line,
            'data': self.data,
            'time': self.time,
        }

    def __getitem__(self, key):
        return self.dict[key]

    def __setitem__(self, key, value):
        self.dict[key] = value

    def __len__(self):
        return len(self.dict)

    def __contains__(self, key):
        return key in self.dict

# 客制化 Logger
class Logger(logging.Logger):
    """
    獲取logger，格式化日志輸出
    ex:
        logger = Logger('tool', 'stream')
        logger.info('Hello')

    >> 2020-03-10 00:36:10,873のINFOのtoolの./tool.py(line:299)のHello
    """

    PATH = sys.path[0] + '/logs/'
    LOG_FILE = '%s%s.log'%(PATH, datetime.now().strftime('%Y%m%d'))
    LOG_FORMAT = '%(asctime)sの%(levelname)sの%(name)sの%(pathname)s(line:%(lineno)d)の%(message)s'

    def __new__(cls, *args, **kwargs):
        obj = super().__new__(cls)
        cls.__init__(obj, *args, **kwargs)
        return obj.logger

    def __init__(self, name='root', type='stream', level='info'):
        """
        :param name(str): Log Name
        :param type(str): Log Type: print, stream, file
        :param level(str): Log Level: critical, error, warn, info, debug
        """
        super().__init__(name, level.upper())
        func = getattr(self, f'{type}_logger')
        self.logger = func(name, level)

    def file_handler(self, logfile=LOG_FILE):
        if not os.path.isdir(self.PATH):
            os.makedirs(self.PATH)
        _format = self.LOG_FORMAT
        handler = logging.FileHandler(logfile)
        formatter = logging.Formatter(_format)
        handler.setFormatter(formatter)
        return handler

    def stream_handler(self):
        _format = self.LOG_FORMAT
        handler = logging.StreamHandler()
        formatter = logging.Formatter(_format)
        handler.setFormatter(formatter)
        return handler

    def print_handler(self):
        _format = '%(message)s'
        handler = logging.StreamHandler()
        formatter = logging.Formatter(_format)
        handler.setFormatter(formatter)
        return handler

    def file_logger(self, obj='__main__', level='info', logfile=LOG_FILE):
        handler = self.file_handler(logfile)
        logger = logging.getLogger(obj)
        logger.setLevel(level.upper())
        logger.addHandler(handler)
        return logger

    def stream_logger(self, obj='__main__', level='info'):
        handler = self.stream_handler()
        logger = logging.getLogger(obj)
        logger.setLevel(level.upper())
        logger.addHandler(handler)
        return logger

    def print_logger(self, obj='__main__', level='info'):
        handler = self.print_handler()
        logger = logging.getLogger(obj)
        logger.setLevel(level.upper())
        logger.addHandler(handler)
        return logger

    def makeRecord(self, name, level, fn, lno, msg, args, exc_info,
                   func=None, extra=None, sinfo=None):
        """
        A factory method which can be overridden in subclasses to create
        specialized LogRecords.
        """
        rv = _logRecordFactory(name, level, fn, lno, msg, args, exc_info, func,
                             sinfo)
        if extra is not None:
            for key in extra:
                # 修正變量無法取代問題
                #if (key in ["message", "asctime"]) or (key in rv.__dict__)
                if (key in ["message", "asctime"]):
                    raise KeyError("Attempt to overwrite %r in LogRecord" % key)
                rv.__dict__[key] = extra[key]
        return rv

if __name__ == '__main__':

    #os.chdir(sys.path[0])
    #logrotate('logs')
    logger = Logger('main')
    logger.info('Hello')



#!/usr/bin/env python

"""
加解密功能
"""

import base64


def pass_encode(user, pwd, split='の'):
    """
    :param user(str): user
    :param pwd(str): password
    :return: key(str)
    """
    string = f'{user}{split}{pwd}'.encode('utf8')
    key = base64.encodebytes(string)
    return key.decode('utf8').strip()

def pass_decode(key, split='の'):
    """
    :param key(str): 
    :return: user(str), pwd(str)
    """

    key = key.encode('utf8')
    string = base64.decodebytes(key)
    string = string.decode('utf8')
    user, pwd = string.split(split)
    return user, pwd

if __name__ == '__main__':

    user = 'walker'
    pwd = 'P@ssw0rd'

    key = pass_encode(user, pwd)
    print(key)
    user, pwd = pass_decode(key)
    print(user, pwd)



#!/usr/bin/env python3
# file: mail.py
# author: walker

"""
Changelogs

2018.06.29: init
2020.04.18: 支援io.BytesIO附件 

"""

import os
import sys
import smtplib
import traceback
from datetime import datetime
from email.mime.text import MIMEText
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart

class Mail(object):

    def __init__(self, host, port, user='', pwd=''):
        """
        :param host(str): 邮件服务器主机
        :param port(int): Port
        :param user(str): 登入帐号
        :param pwd(str): 密码
        """
        self.mail = smtplib.SMTP(host, port)
        try:
            self.mail.starttls()
        except smtplib.SMTPNotSupportedError:
            pass
        if user and pwd:
            self.mail.login(user, pwd)
        
    def send(self, subject, content, _from, _to, _cc=[], _bcc=[], _file=[], _img=[]):
        """
        :param subject(str): 标题
        :param content(str): 邮件内容
        :param _from(str): 发件人
        :param _to(list): 收件人
        :param _cc(list): 抄送
        :param _bcc(list): 密本抄送
        :param _file(list): 附件, [文件名]或是[[文件名, io.BytesIO]]
        :param _img(str): 邮件内容有嵌图片时使用
        """

        if isinstance(_to, str):
            _to = [_to]
        if _cc and isinstance(_cc, str):
            _cc = [_cc]
        if _bcc and isinstance(_bcc, str):
            _bcc = [_bcc]

        msg = MIMEMultipart()
        msg['to'] = ','.join(_to)
        msg['cc'] = ','.join(_cc)
        msg['from'] = _from
        msg['subject'] = subject

        # HTML Content 
        content = MIMEText(content, 'html', 'utf-8')
        msg.attach(content)

        # with Picture
        if _img:
            for n in _img:
                with open(n, 'rb') as f: img = f.read()
                msg_image = MIMEImage(img)
                msg_image.add_header('Content-ID', filename)
                msg.attach(msg_image)

        # Attachment
        for n in _file:
            if isinstance(n, str):
                filename = n
                attach = MIMEText(open(filename, 'rb').read(), 'base64', 'utf-8')
                attach.add_header('Content-Type', 'application/octet-stream')
                attach.add_header('Content-Disposition', 'attachment', filename=filename)
                msg.attach(attach)
            elif isinstance(n, list):
                filename, cache = n
                attach = MIMEText(cache.read(), 'base64', 'utf-8')
                attach.add_header('Content-Type', 'application/octet-stream')
                attach.add_header('Content-Disposition', 'attachment', filename=filename)
                msg.attach(attach)

        _From = _from
        _To = _to + _cc + _bcc

        self.mail.sendmail(_From, _To, msg.as_string())

    def close(self):
        self.mail.quit()

if __name__ == '__main__':

    os.chdir(sys.path[0])

    _from = 'XXXX'
    _to = 'YYYY'

    ## Local
    #mail = Mail(host='127.0.0.1', port=25)
    ## Gmail
    #mail = Mail(host='smtp.gmail.com', port=587, user='XXXX', pwd='XXXXX')

    subject = 'Python Test %s'%(datetime.now().strftime('%Y-%m-%d %H:%M:%S'))
    content = 'This is a Test!!!'

    mail.send(subject, content, _from, _to)




#!/usr/bin/env python

from setuptools import setup, find_packages

setup(
    name = 'viger-tool',
    version = '0.0.15',
    keywords = ['pip', 'tool', 'viger'],
    description = 'Tool package',
    license = 'MIT Licence',

    url = 'https://github.com/viger1228/viger-tool',
    author = 'walker',
    author_email = 'walkerIVI@gmail.com',

    packages = find_packages(),
    include_package_data = True,
    platforms = 'any',
    install_requires = [
        'requests',
        'prettytable'
    ]
)
